/*
    Copyright (C) 2004 Fons Adriaensen <fons.adriaensen@skynet.be>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include <math.h>
#include <time.h>
#include "prbsgen.h"
#include "rngen.h"


const double Rngen::_p31 = 2147483648.0;
const double Rngen::_p32 = 4294967296.0;


Rngen::Rngen (void)
{
  init (0);
}


Rngen::~Rngen (void)
{
}


void Rngen::init (U32 s)
{
  int i, j;
  Prbsgen G1;
  Prbsgen G2;

  if (s == 0) s = time (0);

  G1.set_poly (Prbsgen::G16);
  G2.set_poly (Prbsgen::G32);
  G1.set_stat (s | (s >> 16));
  G2.set_stat (s);

  for (i = 0; i < 55; i++)
  {
    G1.step ();
    j = G1.stat () & 4095;
    while (j--) G2.step ();
    _a [i] = G2.stat ();
  }

  _f = false;
  _i = 0;
}

 
double Rngen::grand (void)
{
  double a, b, r;

  if (_f)
  {
    _f = false;
    return _v;
  }

  do
  {
    a = irand () / _p31 - 1.0;
    b = irand () / _p31 - 1.0;
    r = a * a + b * b;
  }
  while ((r > 1.0) || (r == 0.0));

  r = sqrt (-2.0 * log (r) / r);
  _f = true;
  _v = r * b;

  return r * a;
}    

 
void Rngen::grand (double *x, double *y)
{
  double a, b, r;

  do
  {
    a = irand () / _p31 - 1.0;
    b = irand () / _p31 - 1.0;
    r = a * a + b * b;
  }
  while ((r > 1.0) || (r == 0.0));

  r = sqrt (-log (r) / r);
  *x = r * a;
  *y = r * b;
}    


void Rngen::grand (float *x, float *y)
{
  double a, b, r;

  do
  {
    a = irand () / _p31 - 1.0;
    b = irand () / _p31 - 1.0;
    r = a * a + b * b;
  }
  while ((r > 1.0) || (r == 0.0));

  r = sqrt (-log (r) / r);
  *x = r * a;
  *y = r * b;
}    

